# Copyright (C) 2012 Linaro
#
# This file is part of linaro-ci-dashboard.
#
# linaro-ci-dashboard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# linaro-ci-dashboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-ci-dashboard.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from frontend.models.loop import Loop
from jenkinsserver.models.jenkins_server import JenkinsServer


# The internal names for the different build type, and a user visible name
BUILD_ANDROID = (r'build-android', 'Android')
BUILD_ANDROID_RESTRICTED = (r'build-android-restricted', 'Android Restricted')
BUILD_ANDROID_TOOLCHAIN = (r'build-android-toolchain', 'Android Toolchain')
BUILD_ANDROID_TOOLCHAIN_LINARO = (r'build-android-toolchain-linaro',
                                    'Android Toolchain Linaro')
# All the supported build types.
ANDROID_BUILD_TYPES = (
    BUILD_ANDROID,
    BUILD_ANDROID_RESTRICTED,
    BUILD_ANDROID_TOOLCHAIN,
    BUILD_ANDROID_TOOLCHAIN_LINARO,
    )

# Internal name of the board, and a user visible name
BEAGLEBOARD = (r'beagleboard', 'Beagleboard')
PANDABOARD = (r'panda', 'Pandaboard')
SNOWBALL = (r'snowball', 'Snowball')
# The supported boards for the build.
TARGET_BOARDS = (BEAGLEBOARD, PANDABOARD, SNOWBALL)


class AndroidLoop(Loop):
    """Defines the base class for an Android integration loop."""
    class Meta:
        app_label = 'android_build'

    # The default file name for the manifest file.
    DEFAULT_MANIFEST_FILE = 'default.xml'
    DEFAULT_REPO = 'git://android.git.kernel.org/platform/manifest.git'
    DEFAULT_BRANCH = 'master'
    DEFAULT_BINUTILS = '2.20.1'
    DEFAULT_GCC = '4.4.0'
    DEFAULT_IMAGE_SIZE = '2G'

    # BUILD_TYPE
    build_type = models.CharField(max_length=100,
                                  choices=ANDROID_BUILD_TYPES,
                                  verbose_name='Build type')
    # MANIFEST_REPO
    manifest_repo = models.CharField(max_length=255,
                                     default=DEFAULT_REPO,
                                     verbose_name='Manifest repo')
    # MANIFEST_BRANCH
    manifest_branch = models.CharField(max_length=50, default=DEFAULT_BRANCH)
    # REPO_QUIET
    repo_quiet = models.BooleanField(default=False)
    # MANIFEST_FILENAME
    manifest_filename = models.CharField(max_length=255,
                                         default=DEFAULT_MANIFEST_FILE)
    # MAKE_TARGETS
    make_targets = models.CharField(max_length=255, blank=True)
    # MAKE_JOBS
    make_jobs = models.PositiveIntegerField(default=0)
    # RAMDISK_SIZE
    ramdisk_size = models.PositiveIntegerField(default=0)
    # EXTERNAL_TARBALL
    external_tarball = models.CharField(max_length=255, blank=True)
    # BUILD_FS_IMAGE
    build_fs_image = models.BooleanField(default=False,
                                         verbose_name='Build filesystem image')
    # FS_IMAGE_SIZE
    fs_image_size = models.CharField(max_length=20,
                                     blank=True,
                                     verbose_name='Filesystem image size',
                                     default=DEFAULT_IMAGE_SIZE)
    # Users might want to define their own values.
    user_defined_values = models.TextField(blank=True)

    # Data needed for build of type build-android.
    toolchain_url = models.CharField(max_length=255, blank=True)
    target_product = models.CharField(max_length=50, choices=TARGET_BOARDS,
                                        blank=True)
    target_tools_prefix = models.CharField(max_length=255, blank=True)

    # Data needed for build of type build-android-toolchain.
    binutils_version = models.CharField(max_length=25,
                                        default=DEFAULT_BINUTILS)
    binutils_url = models.CharField(max_length=255, blank=True)

    gcc_version = models.CharField(max_length=25, default=DEFAULT_GCC)
    gcc_url = models.CharField(max_length=255, blank=True)

    # TODO need to check the default value for this one.
    sysroot_ndk_url = models.CharField(max_length=255, blank=True)

    # Data needed for LAVA integration.
    lava_submit = models.BooleanField(default=False)
    # TODO need to know the values here
    lava_test_plan = models.CharField(max_length=255, blank=True)
    lava_submit_fatal = models.BooleanField(default=True, blank=True)
    lava_android_binaries = models.BooleanField(default=True, blank=True)

    def save(self, *args, **kwargs):
        self.server = JenkinsServer.objects.get(id=1)
        self.type = self.__class__.__name__
        super(self.__class__, self).save(*args, **kwargs)
        # We do not want a failure in remote server to affect the
        # CI Loop flow.
        try:
            self.server.create_or_update_loop(self)
        except:
            # TODO: Log error.
            pass

    def schedule_build(self):
        parameters = {'parameter': [{'delay': '0'}]}
        return super(self.__class__, self).schedule_build(parameters)
