# Copyright (C) 2012 Linaro
#
# This file is part of linaro-ci-dashboard.
#
# linaro-ci-dashboard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# linaro-ci-dashboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-ci-dashboard.  If not, see <http://www.gnu.org/licenses/>.
# Django settings for dashboard project.

import os
import sys
import shutil
import shlex
import urllib2
import urlparse
from django.conf import settings
from django.core import management
from django.core.management.base import BaseCommand, CommandError
from jenkinsserver.models.jenkins_server import JenkinsServer
from subprocess import check_output, CalledProcessError, Popen, STDOUT


class Command(BaseCommand):
    args = 'start or stop <pid>'
    help = 'Starts/Stops Jenkins service on machine.\n'\
            'You need to specify pid of the jenkins process to stop'

    def __init__(self):
        self.jenkins_war = None
        self.jenkins_pid = None
        self.port = settings.JENKINS_PORT
        self.jenkins_user = settings.JENKINS_ADMIN_USER
        self.jenkins_passwd = settings.JENKINS_ADMIN_PASSWD
        self.jenkins_home = settings.JENKINS_HOME
        self.jenkins_log = settings.JENKINS_LOG_NAME
        self.jenkins_war_url = settings.JENKKINS_WAR_URL
        self.jenkins_id = settings.JENKINS_DB_ID

    def handle(self, *args, **options):
        try:
            if len(args) < 1:
                sys.stdout.write("Need to specify either start/stop\n")
                raise

            if args[0] == 'start':
                self.install_run_jenkins()
                if self.jenkins_pid:
                    msg = "Successfully configured jenkins with pid %s. \n"\
                           % self.jenkins_pid
                    self.stdout.write(msg)

                jenkins_url = 'http://127.0.0.1:%s/jenkins' % self.port
                msg = "Use '%s' in your browser \n" % jenkins_url
            elif args[0] == 'stop':
                if len(args[1:]):
                    for pid in args[1:]:
                        self.stop_jenkins(pid)
                        msg = 'Successfully terminated jenkins with pid %s\n' \
                          % pid
                else:
                    msg = "Missing pid value. "\
                      "Specify pid after stop argument\n"
            elif args[0] == 'sync':
                server = JenkinsServer.objects.get(id=self.jenkins_id)
                server.sync_jobs_and_builds()
                msg = "Successfully synced jenkins and CI loop builds. \n"
            else:
                msg = "Invalid option. Supports only start/stop or sync.\n"

            self.stdout.write(msg)
        except:
            sys.stdout.write("Error processing the command\n")

    def install_run_jenkins(self):
        """
        Installs/configure/start the jenkins making sure another
        instance of jenkins is not running on the same self.port.
        """
        try:
            # Download openid plugin
            cmd = 'mkdir -p %s' % self.jenkins_home
            self.run(cmd)

            # Stop any jenkins which is already running on the same port
            cmd = "lsof -w -n -i tcp:%s | awk -F ' ' '{print $2}' | " \
            "tail -n 1 | sed '/^$/d'" % self.port
            pid = self.run(cmd)
            if pid:
                msg = "Jenkins with pid '%s' is already running on port %s\n" \
                      % (pid.strip(), self.port)
                self.stdout.write(msg)
                return

            # start jenkins service
            self.jenkins_start()

            cmd = 'cp %s %s' % (settings.JENKINS_CONFIG, self.jenkins_home)
            self.run(cmd)

        except CalledProcessError:
            raise CalledProcessError(1, cmd, "Command Error")

    def stop_jenkins(self, pid):
        """
        Stops the jenkins process running with the pid specified.
        param :
        pid: pid of the jenkins service to stop
        """
        cmd = 'kill -9 %s' % pid
        try:
            self.run(cmd)
        except CalledProcessError:
            raise CalledProcessError(1, cmd, "Command Error")

    def download(self, url, filename):
        """
        Downloads the jenkins.war file if it is not already
        present in the JENKINS_HOME dir
        param:
        filename: contains the location of the jenkins.war file
        """
        try:
            response = urllib2.urlopen(urllib2.quote(url, safe=":/"))
            if not (os.path.exists(filename) and  os.path.isfile(filename)):
                fd = open(filename, 'wb')
                shutil.copyfileobj(response, fd)
                fd.close()

            actual_size = int(response.headers['Content-Length'])
            downloaded_size = int(os.stat(filename).st_size)
            response.close()
            if actual_size != downloaded_size:
                raise
        except:
            raise CommandError("Could not retrieve %s" % url)
            sys.exit(1)
        return 0

    def jenkins_start(self):
        """
        Start the jenkins service using the downloaded jenkins.war
        available in the JENKINS_HOME dir.
        Jenkins is started with admin JENKINS_USER/JENKINS_PASSWD
        credentials. Copy the template config.xml which enables
        options for user to login with JENKINS_USER/JENKINS_PASSWD credentials
        from browser.
        """
        # Download Jenkins war file
        self.jenkins_war = os.path.join(self.jenkins_home,
                           os.path.basename(self.jenkins_war_url))

        self.download(self.jenkins_war_url, self.jenkins_war)

        # Start the jenkins service on port 9090
        cmd = '/usr/bin/java -jar %s --httpPort=%s --prefix=/jenkins \
              --argumentsRealm.passwd.%s=%s \
              --argumentsRealm.roles.%s=admin' \
              % (self.jenkins_war, self.port,
                 self.jenkins_user, self.jenkins_passwd,
                 self.jenkins_user)

        cmd = shlex.split(cmd)
        log_fn = os.path.join(self.jenkins_home, self.jenkins_log)
        log_fd = open(log_fn, 'a')
        ret = Popen(cmd, stdout=log_fd, stderr=STDOUT,
                    env={"JENKINS_HOME": self.jenkins_home})

        if ret.returncode:
            raise CalledProcessError(1, cmd, 'cmd Failed')
        log_fd.close()

        # Copy admin user config.xml file to users dir.
        user_dir = self.jenkins_home + '/users/' + self.jenkins_user
        cmd = 'mkdir -p %s' % user_dir
        self.run(cmd)
        cmd = 'cp %s %s' % (settings.JENKINS_USER_CONFIG, user_dir)
        self.run(cmd)

        self.jenkins_pid = ret.pid
        return self.jenkins_pid

    def run(self, cmd):
        try:
            ret = check_output(cmd, shell=True)
            return ret
        except CalledProcessError:
            raise CalledProcessError(1, cmd, "Command Error")
