#!/usr/bin/env python
# Copyright (C) 2012 Linaro
#
# This file is part of linaro-ci-dashboard.
#
# linaro-ci-dashboard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# linaro-ci-dashboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-ci-dashboard.  If not, see <http://www.gnu.org/licenses/>.

import base64
from django.db import models
from jenkinsserver.models.jenkins_server import JenkinsServer
from lib.logger import Logger


class Loop(models.Model):
    class Meta:
        app_label = 'frontend'

    name = models.CharField(max_length=200, unique=True)
    server = models.ForeignKey(JenkinsServer,
                               default=None,
                               null=True,
                               on_delete=models.SET_NULL)
    type = models.CharField(max_length=50, editable=False)
    is_restricted = models.BooleanField(default=False,
                                        verbose_name='Build is restricted')
    is_official = models.BooleanField(default=False,
                                      verbose_name='Build is official')
    next_loop = models.OneToOneField('self', blank=True, null=True,
                                     unique=True, on_delete=models.SET_NULL,
                                     related_name='previous_loop',
                                     verbose_name='Next loop in chain')

    # List of fields to be excluded by default from the base64 encoded config.
    # If a subclass needs different fields, it is necessary to override this
    # variable.
    EXCLUDE_LIST = ['id', 'name', 'server', 'loop_ptr']

    def __init__(self, *args, **kwargs):
        self.log = Logger.getClassLogger(self)
        super(Loop, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        super(Loop, self).save(*args, **kwargs)

    def schedule_build(self, parameters=None):
        '''Add a LoopBuild record for this Loop.

        Also try to send a build job signal to the server, but if it fails,
        ignore it.
        '''

        # Every time before the build is scheduled, update the
        # loop config on the remote server.
        self.server.create_or_update_loop(self)

        from frontend.models.loop_build import LoopBuild
        build = LoopBuild()
        build.loop = self
        build.status = 'scheduled'
        build.duration = 0.00
        build.save()
        try:
            build.remote_number = self.server.schedule_job_build(self.name,
                                                                 parameters)
            build.save()
        except:
            # TODO: Log error.
            pass
        return build

    def json(self):
        """Return Loop in a format suitable for serialization as JSON. The
        method is called without underscores to be callabel from Django
        templates.
        """
        from django.forms.models import model_to_dict
        return model_to_dict(self)

    @staticmethod
    def can_chain_into():
        """Retrieves the list of loops that this one can be chained with.
        Subclasses need to override this and provide their own chainable loops
        list.

        Direction of the chain is to be intended from left to right in an
        origin<>target way. Origin is always 'this' loop instance.

        :return List of loops it can chain into.
        """
        return []

    def __repr__(self):
        return "%s(id=%s, ...)" % (self.__class__.__name__, self.id)

    def base64_config(self):
        """ Return loop configuration as base64 encoded string.
        """
        text_config = u''
        for field in self._meta.fields:
            if not field.name in self.EXCLUDE_LIST:
                text_config += u'%s=%s\n' % (field.name.upper(),
                                             field.value_to_string(self))
        return base64.b64encode(text_config)

    def get_child_object(self):
        """Returns the instance of an actual child loop."""
        loop_type = self.type.lower()
        return self.__getattribute__(loop_type)

    @classmethod
    def get_all_chainable(cls):
        """Returns all loop objects that can be chained to this one."""
        result = []
        for loop_type in cls.can_chain_into():
            result.extend(Loop.objects.filter(type=loop_type))
        return result
