# Copyright (C) 2012 Linaro
#
# This file is part of linaro-ci-dashboard.
#
# linaro-ci-dashboard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# linaro-ci-dashboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-ci-dashboard.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from frontend.models.loop import Loop


# Default delimiter to separate values from keys in the text field.
DEFAULT_DELIMITER = '='


class TextFieldLoop(Loop):
    """
    This is the base class model for the loops that want to be a free-form loop
    with just a text field to insert values.
    A loop of this kind is not much useful, it needs to be chained with another
    one.

    This model in particular is an abstract model, it is necessary to inherits
    from it to be used.
    """
    class Meta:
        app_label = 'frontend'
        abstract = True

    values = models.TextField()

    def schedule_build(self, parameters=None):
        return None

    def values_to_dict(self):
        """
        Returns a dictionary representation of the values inserted. The
        key<>value pairs are split on DEFAULT_DELIMITER.

        :return A dictionary of the values inserted.
        """
        text_to_dict = {}
        valid, lines = self.valid_values(self.values)

        if valid:
            for line in lines:
                key, value = line.split(DEFAULT_DELIMITER)
                text_to_dict[key.lower()] = value

        return text_to_dict

    @staticmethod
    def valid_values(values):
        """
        Checks if the values inserted by the user are well formed, meaning that
        each key<>value pair is using the correct delimiter.

        :param values: the string with all the key<>value pairs, separated with
        a newline character.
        :type str
        :return a boolean for the validity, and the list of lines.
        """
        valid = True
        lines = values.splitlines()
        for line in lines:
            if DEFAULT_DELIMITER in line:
                continue
            else:
                valid = False
                break
        return valid, lines
