#!/usr/bin/env python
# Copyright (C) 2012 Linaro
#
# This file is part of linaro-ci-dashboard.
#
# linaro-ci-dashboard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# linaro-ci-dashboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-ci-dashboard.  If not, see <http://www.gnu.org/licenses/>.

import frontend.management.commands.jenkins as jenkins
import frontend.management.commands.setsuperuser as setsuperuser
from django.contrib.auth.models import User
from django.core.management.base import CommandError
from django.conf import settings
from subprocess import  CalledProcessError
from django.core import management
from StringIO import StringIO
from django.test import TestCase
import os
import sys
import shutil


class SetsuperuserCommandTest(TestCase):

    def setUp(self):
        self.command = setsuperuser.Command()

    def test_find_and_update_user_non_existing(self):
        self.assertRaises(CommandError,
                          self.command.find_and_update_user,
                          ("non_existing_user"))

    def test_find_and_update_user(self):
        user = User(username="existing_user")
        user.save()
        self.command.find_and_update_user("existing_user")
        user = User.objects.get(username="existing_user")
        self.assertEquals(user.is_staff, True)
        self.assertEquals(user.is_superuser, True)


class JenkinsCommandTest(TestCase):

    orig_dir = os.getcwd()

    def setUp(self):
        self.command = jenkins.Command()
        self.command.port = settings.JENKINS_PORT
        self.url = 'http://updates.jenkins-ci.org/latest/bazaar.hpi'
        os.environ["JENKINS_HOME"] = os.path.join(self.orig_dir, 'tmp/',
                                     'jenkins_home')
        self.command.jenkins_home = os.getenv("JENKINS_HOME")
        self.command.jenkins_war = os.path.join(self.command.jenkins_home,
                              os.path.basename(self.command.jenkins_war_url))
        self.download_filename = os.path.join(self.command.jenkins_home,
                                 os.path.basename(self.url))

        if not os.path.exists(self.command.jenkins_home):
            os.makedirs(self.command.jenkins_home)

        super(JenkinsCommandTest, self).setUp()

    def tearDown(self):
        os.chdir(self.orig_dir)
        if os.path.exists(self.download_filename):
            cmd = 'rm  %s' % self.download_filename
            self.command.run(cmd)

        parent_dir = os.path.dirname(self.command.jenkins_home)
        if os.path.isdir(parent_dir):
            shutil.rmtree(parent_dir)

        super(JenkinsCommandTest, self).tearDown()

    def test_download_non_existing_package(self):
        url = 'http://mirrors.jenkins-ci.org/war/latest/jenkins1.war'
        filename = os.path.join('/tmp/', os.path.basename(url))
        self.assertRaises(CommandError, self.command.download, url, filename)

    def test_download_existing_package(self):
        ret = self.command.download(self.url, self.download_filename)
        self.assertTrue(ret == 0)

    def test_install_run_jenkins(self):
        orig_stdout = sys.stdout
        stdout = sys.stdout = StringIO()
        cmd = "lsof -w -n -i tcp:%s | awk -F ' ' '{print $2}' | tail -n 1 | "\
               "sed '/^$/d'" % self.command.port
        self.command.jenkins_pid = self.command.run(cmd)
        management.call_command('jenkins', 'start', interactive=False)
        if len(self.command.jenkins_pid) != 0:
            msg = "Jenkins with pid '%s' is already running on port %s"\
                  % (self.command.jenkins_pid.strip(), self.command.port)
        else:
            msg = "Successfully configured jenkins with pid"
        sys.stdout = orig_stdout
        stdout.seek(0)
        self.assertIn(msg, stdout.read())

    def test_invalid_run(self):
        cmd = 'rm /tmp/dummy'
        self.assertRaises(CalledProcessError, self.command.run, cmd)

    def test_valid_run(self):
        ret = self.command.run(['whoami'])
        self.assertTrue(ret.strip() == os.getenv('USER'))

    def test_jenkins_stop_jenkins_non_existing_pid(self):
        pid = '123456'
        self.assertRaises(CalledProcessError, self.command.stop_jenkins, pid)

    def test_jenkins_stop_jenkins(self):
        orig_stdout = sys.stdout
        stdout = sys.stdout = StringIO()
        cmd = "lsof -w -n -i tcp:%s | awk -F ' ' '{print $2}' | tail -n 1 | "\
               "sed '/^$/d'" % self.command.port
        pid = self.command.run(cmd)
        management.call_command('jenkins', 'start', interactive=False)
        if len(pid) != 0:
            msg = "Jenkins with pid '%s' is already running on port %s"\
                % (pid.strip(), self.command.port)
        else:
            pid = self.command.jenkins_start()
            management.call_command('jenkins', 'stop', pid, interactive=False)
            msg = "Successfully terminated jenkins with pid"
        sys.stdout = orig_stdout
        stdout.seek(0)
        self.assertIn(msg, stdout.read())

    def test_jenkins_stop_missing_value(self):
        orig_stdout = sys.stdout
        stdout = sys.stdout = StringIO()
        management.call_command('jenkins', 'stop', interactive=False)
        msg = "Missing pid value. Specify pid after stop argument"
        sys.stdout = orig_stdout
        stdout.seek(0)
        self.assertIn(msg, stdout.read())

    def test_jenkins_invalid_option(self):
        orig_stdout = sys.stdout
        stdout = sys.stdout = StringIO()
        management.call_command('jenkins', 'Invalid', interactive=False)
        msg = "Invalid option. Supports only start/stop."
        sys.stdout = orig_stdout
        stdout.seek(0)
        self.assertIn(msg, stdout.read())

    def test_jenkins_invalid_option(self):
        orig_stdout = sys.stdout
        stdout = sys.stdout = StringIO()
        management.call_command('jenkins', interactive=False)
        msg = "Need to specify either start/stop"
        sys.stdout = orig_stdout
        stdout.seek(0)
        self.assertIn(msg, stdout.read())
